@@ -214,6 +214,7 @@ class SimpleTest(TestCase):
         self.assertEqual(judy.color, Color.green)
         self.assertEqual(judy.music, MusicGenre.banjo)
         self.assertEqual(judy.sport, Sports.poker)
+        self.assertEqual(judy.nullable, None)
         judy.color = Color.blue
         judy.music = MusicGenre.rock
         judy.sport = Sports.mountaineering
@@ -234,6 +235,9 @@ class SimpleTest(TestCase):
         tom.save()
         tom2 = Favourites.objects.get(name='Tom')
         self.assertEqual(tom2.color, Color.red)
+        tom_and_judy = Favourites.objects.filter(color__in=(Color.blue, Color.red))
+        self.assertEqual(tom_and_judy.count(), 2)
+        self.assertEqual({obj.name for obj in tom_and_judy}, {'Judy', 'Tom'})
 
     def test_form_with_choicefields(self):
         from dj._choicestestproject.app.models import Favourites, Color,\
@@ -245,6 +249,7 @@ class SimpleTest(TestCase):
         self.assertEqual(empty_form._bound_value('music'), None)
         self.assertEqual(empty_form._bound_value('sport'), None)
         self.assertEqual(empty_form._bound_value('name'), None)
+        self.assertEqual(empty_form._bound_value('nullable'), None)
         empty_form_data = FavouritesForm(data={
                 'color': 1, 'music': 2, 'sport': 3, 'name': 'Richard'})
         self.assertTrue(empty_form_data.is_valid())
@@ -253,6 +258,7 @@ class SimpleTest(TestCase):
                 MusicGenre.country.id)
         self.assertEqual(empty_form_data._bound_value('sport'), Sports.baseball.id)
         self.assertEqual(empty_form_data._bound_value('name'), 'Richard')
+        self.assertEqual(empty_form_data._bound_value('nullable'), None)
         judy = Favourites.create(name='Judy')
         judy_form = FavouritesForm(instance=judy)
         self.assertFalse(judy_form.is_valid()) # because it's not bound
@@ -260,11 +266,49 @@ class SimpleTest(TestCase):
         self.assertEqual(judy_form._bound_value('music'), MusicGenre.banjo.id)
         self.assertEqual(judy_form._bound_value('sport'), Sports.poker.id)
         self.assertEqual(judy_form._bound_value('name'), 'Judy')
+        self.assertEqual(judy_form._bound_value('nullable'), None)
         data_for_form = dict(judy_form.initial)
-        data_for_form.update({'color': 3})
+        data_for_form.update({'nullable': Color.red})
         judy_form_data = FavouritesForm(instance=judy, data=data_for_form)
         self.assertTrue(judy_form_data.is_valid())
-        self.assertEqual(judy_form_data._bound_value('color'), Color.blue.id)
+        self.assertEqual(judy_form_data._bound_value('color'), Color.green.id)
         self.assertEqual(judy_form_data._bound_value('music'), MusicGenre.banjo.id)
         self.assertEqual(judy_form_data._bound_value('sport'), Sports.poker.id)
         self.assertEqual(judy_form_data._bound_value('name'), 'Judy')
+        self.assertEqual(judy_form_data._bound_value('nullable'), Color.red.id)
+        invalid_data_for_form = dict(judy_form.initial)
+        invalid_data_for_form.update({'color': -1, 'music': '', 'sport': None,
+                'nullable': ''})
+        judy_form_invalid_data = FavouritesForm(instance=judy,
+                data=invalid_data_for_form)
+        self.assertFalse(judy_form_invalid_data.is_valid())
+        self.assertEqual(set(judy_form_invalid_data.errors.keys()), {'color',
+            'music', 'sport'})
+        self.assertTrue('not a valid choice' in
+                judy_form_invalid_data.errors['color'][0])
+        self.assertTrue('cannot be null' in
+                judy_form_invalid_data.errors['music'][0])
+        self.assertTrue('cannot be null' in
+                judy_form_invalid_data.errors['sport'][0])
+
+    def test_regularintegers(self):
+        from dj._choicestestproject.app.models import Color, RegularIntegers
+        from dj._choicestestproject.app.forms import RegularIntegersForm
+        rint = RegularIntegers.create()
+        self.assertEqual(rint.color, Color.green.id)
+        rint_invalid = RegularIntegers.create(color=-1)
+        self.assertEqual(rint_invalid.color, -1)
+        empty_form = RegularIntegersForm()
+        self.assertFalse(empty_form.is_valid()) # because it's not bound
+        valid_form = RegularIntegersForm(data={'color': 1})
+        self.assertTrue(valid_form.is_valid())
+        invalid_form = RegularIntegersForm(data={'color': -1})
+        self.assertFalse(invalid_form.is_valid())
+        self.assertTrue('color' in invalid_form.errors)
+        self.assertTrue('Select a valid choice' in
+                invalid_form.errors['color'][0])
+        invalid_type_form = RegularIntegersForm(data={'color': 'aaa'})
+        self.assertFalse(invalid_type_form.is_valid())
+        self.assertTrue('color' in invalid_type_form.errors)
+        self.assertTrue('Select a valid choice' in
+                invalid_type_form.errors['color'][0])
